import a9a
import sys
import pathlib

source, target = map(pathlib.Path, sys.argv[1:3])
if source.is_file():
    with open(source, "rb") as f:
        bts = f.read()
    a9a.Archive.from_bytes(bts).to_directory(target)
else:
    with open(target, "wb") as f:
        f.write(a9a.Archive.from_directory(source).to_bytes())



DIRECTORY = 0
EXECUTABLE = 1
FILE = 2
LINK = 3


def encode_lines(content):
    lines = content.split(b"\n")
    last_line = b"\n\\" + lines[-1] + b"\n"
    return b"".join(b"\n " + l for l in lines[:-1]) + last_line


def encode_nodes(nodes):
    return b"".join(encode_node(n, t, c) for n, (t, c) in sorted(nodes.items()))


def encode_file(name, content):
    return b":" + name + b"/" + encode_lines(content)


def encode_executable(name, content):
    return b"!" + name + b"/" + encode_lines(content)


def encode_link(name, content):
    return b"@" + name + b"/" + encode_lines(content)


def encode_directory(name, content):
    return b"/" + name + b"/\n" + encode_nodes(content) + b"\\\n"


def encode_node(name, typ, content):
    if typ == DIRECTORY:
        return encode_directory(name, content)
    if typ == EXECUTABLE:
        return encode_executable(name, content)
    if typ == FILE:
        return encode_file(name, content)
    if typ == LINK:
        return encode_link(name, content)
    assert False



import pathlib
import stat

DIRECTORY = 0
EXECUTABLE = 1
FILE = 2
LINK = 3


def write_node(path, node):
    path = pathlib.Path(path)
    typ, content = node
    if typ == LINK:
        write_link(path, content)
    elif typ == DIRECTORY:
        write_directory(path, content)
    elif typ == EXECUTABLE:
        write_executable(path, content)
    elif typ == FILE:
        write_file(path, content)
    else:
        assert False


def write_link(path, content):
    path = pathlib.Path(path)
    path.symlink_to(content)


def write_directory(path, nodes):
    path = pathlib.Path(path)
    path.mkdir()
    for name, node in nodes.items():
        write_node(path / name.decode(), node)


def write_executable(path, content):
    path = pathlib.Path(path)
    with open(path, "wb") as f:
        f.write(content)
    path.chmod(path.stat().st_mode | stat.S_IXUSR | stat.S_IXOTH | stat.S_IXGRP)


def write_file(path, content):
    path = pathlib.Path(path)
    with open(path, "wb") as f:
        f.write(content)



import pathlib

DIRECTORY = 0
EXECUTABLE = 1
FILE = 2
LINK = 3


def read_node(path):
    path = pathlib.Path(path)
    if path.is_symlink():
        return read_link(path)
    if path.is_dir():
        return read_directory(path)
    if not path.is_file():
        assert False
    mode = path.stat().st_mode
    mode, o = divmod(mode, 8)
    mode, g = divmod(mode, 8)
    mode, u = divmod(mode, 8)
    if u & 1:
        return read_executable(path)
    return read_file(path)


def read_link(path):
    path = pathlib.Path(path)
    return (LINK, bytes(path.readlink()))


def read_directory(path):
    path = pathlib.Path(path)
    nodes = {p.name.encode(): read_node(p) for p in path.iterdir()}
    return (DIRECTORY, nodes)


def read_executable(path):
    path = pathlib.Path(path)
    with path.open("rb") as f:
        content = f.read()
    return (EXECUTABLE, content)


def read_file(path):
    path = pathlib.Path(path)
    with path.open("rb") as f:
        content = f.read()
    return (FILE, content)



import setuptools

with open("../readme.md", "r") as f:
    long_description = f.read()

setuptools.setup(
    name="a9a",
    version="0.0.2",
    author="Uladzislau Khamkou",
    description="a9a archivator",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    python_requires=">=3.6",
    py_modules=["a9a"],
    package_dir={"": "."},
    install_requires=[],
)



import a9a.encoder
import a9a.decoder
import a9a.dir_reader
import a9a.dir_writer


class Archive:
    def __init__(self, nodes=None):
        if nodes is None:
            nodes = {}
        self.content = nodes

    def to_bytes(self):
        return b"a9a\n" + encoder.encode_nodes(self.content)

    def __repr__(self):
        return f"Archive({self.content})"

    @staticmethod
    def from_bytes(bts):
        assert bts[:4] == b"a9a\n"
        return Archive(decoder.decode_nodes(bts[4:])[0])

    def to_directory(self, dir_path):
        return dir_writer.write_directory(dir_path, self.content)

    @staticmethod
    def from_directory(dir_path):
        return Archive(dir_reader.read_directory(dir_path)[1])



DIRECTORY = 0
EXECUTABLE = 1
FILE = 2
LINK = 3


def decode_name(bts, start=0):
    i = start
    while bts[i] != b"/"[0]:
        i += 1
    return bts[start:i], i


def decode_line(bts, start=0):
    i = start
    while bts[i] != b"\n"[0]:
        i += 1
    return bts[start:i], i


def decode_lines(bts, start=0):
    assert bts[start] == b"\n"[0]
    lines, i = [], start
    while bts[i + 1] != b"\\"[0]:
        line, i = decode_line(bts, i + 2)
        lines.append(line)
    line, i = decode_line(bts, i + 2)
    assert bts[i] == b"\n"[0]
    lines.append(line)
    return b"\n".join(lines), i + 1


def decode_nodes(bts, start=0):
    i, nodes = start, {}
    while i < len(bts) and bts[i] != b"\\"[0]:
        (name, typ, content), i = decode_node(bts, i)
        nodes[name] = (typ, content)
    return nodes, i


def decode_file(bts, start=0):
    assert bts[start] == b":"[0]
    name, i = decode_name(bts, start + 1)
    content, j = decode_lines(bts, i + 1)
    return (name, FILE, content), j


def decode_executable(bts, start=0):
    assert bts[start] == b"!"[0]
    name, i = decode_name(bts, start + 1)
    content, j = decode_lines(bts, i + 1)
    return (name, EXECUTABLE, content), j


def decode_link(bts, start=0):
    assert bts[start] == b"@"[0]
    name, i = decode_name(bts, start + 1)
    content, j = decode_lines(bts, i + 1)
    return (name, LINK, content), j


def decode_directory(bts, start=0):
    assert bts[start] == b"/"[0]
    name, i = decode_name(bts, start + 1)
    assert bts[i + 1] == b"\n"[0]
    nodes, j = decode_nodes(bts, i + 2)
    assert bts[j : j + 2] == b"\\\n"
    return (name, DIRECTORY, nodes), j + 2


def decode_node(bts, start=0):
    if bts[start] == b"/"[0]:
        return decode_directory(bts, start)
    if bts[start] == b"@"[0]:
        return decode_link(bts, start)
    if bts[start] == b"!"[0]:
        return decode_executable(bts, start)
    if bts[start] == b":"[0]:
        return decode_file(bts, start)
    assert False
